"""\
Contains the core objects used by the other drivers.

Tips for implementers:

* By having a ``connection_name`` attribute on the service you should be able
  to use the same service object with different types of drivers. For example
  an LDAP driver might use a the connection service ``service.ldap`` so by
  specifying ``ldap`` as the connection name when instantiating the service,
  you can access the LDAP connection as
  ``service[service[name].connection_name]`` and thus use the un-modified
  service.
* Could do with read connections as well as write connections?

ToDo
====

* Change the connect and release functions to use the service instead of self

"""

import md5 as _md5

#
# Encryption Functions
#

def md5(password, secret=''):
    result = _md5.md5(password)
    result.update(secret)
    result = result.hexdigest()
    return result

#
# Exceptions
#

try:
    from authkit import AuthKitNoSuchUserError, AuthKitNoSuchRoleError, \
       AuthKitNoSuchGroupError, AuthKitNotSupportedError, AuthKitError
except ImportError:
    class NoSuchUserError(Exception):
        pass
        
    class NoSuchRoleError(Exception):
        pass
    
    class NoSuchGroupError(Exception):
        pass
        
    class NotSupportedError(Exception):
        pass

    class UserManagerError(Exception):
        pass
else:
    class NoSuchUserError(AuthKitNoSuchUserError):
        pass
        
    class NoSuchRoleError(AuthKitNoSuchRoleError):
        pass
    
    class NoSuchGroupError(AuthKitNoSuchGroupError):
        pass
        
    class NotSupportedError(AuthKitNotSupportedError):
        pass

    class UserManagerError(AuthKitError):
        pass

