import logging

from bn import AttributeDict

log = logging.getLogger(__name__)

class ProxyingAttributeDict(AttributeDict):
    def __init__(self, flow, module, oself, *k, **p):
        self['module'] = module
        self['flow'] = flow
        self['oself'] = oself
        #AttributeDict.__init__(self, *k, **p)

    def __getattr__(self, name):
        if hasattr(self['module'], name):
            def func(*k, **p):
                if not self['flow'].has_key(self['oself'].connection_name):
                    raise Exception(
                        'The %rservice has not been started'%(
                            self['oself'].connection_name
                        )
                    )
                return getattr(self['module'], name)(self['flow'], *k, **p)
            return func
        elif hasattr(self['oself'], name):
            return self['oself'].__dict__[name]
        else:
            raise AttributeError('No such attribute %r'%name)

class UserService(object):
    def __init__(self, module, connection_name='database', encrypt=None):
        self.module = module
        self.connection_name = connection_name

    def start(self, flow, key):
        flow[key] = ProxyingAttributeDict(flow, self.module, self)

class BaseUserService(UserService):

    @staticmethod
    def config(flow, name):
        return {}

class PostgresqlUserService(BaseUserService):
    @staticmethod
    def create(flow, name, config=None):
        import usermanager.driver.postgresql
        return PostgresqlUserService(usermanager.driver.postgresql)

class SQLiteUserService(BaseUserService):
    pass

